@@ -222,6 +222,46 @@ class Database(common.BaseObject):
         return [manipulator.__class__.__name__
                 for manipulator in self.__outgoing_copying_manipulators]
 
+    def with_options(self, codec_options=None, read_preference=None,
+                     write_concern=None, read_concern=None):
+        """Get a clone of this database changing the specified settings.
+
+          >>> db1.read_preference
+          Primary()
+          >>> from pymongo import ReadPreference
+          >>> db2 = db1.with_options(read_preference=ReadPreference.SECONDARY)
+          >>> db1.read_preference
+          Primary()
+          >>> db2.read_preference
+          Secondary(tag_sets=None)
+
+        :Parameters:
+          - `codec_options` (optional): An instance of
+            :class:`~bson.codec_options.CodecOptions`. If ``None`` (the
+            default) the :attr:`codec_options` of this :class:`Collection`
+            is used.
+          - `read_preference` (optional): The read preference to use. If
+            ``None`` (the default) the :attr:`read_preference` of this
+            :class:`Collection` is used. See :mod:`~pymongo.read_preferences`
+            for options.
+          - `write_concern` (optional): An instance of
+            :class:`~pymongo.write_concern.WriteConcern`. If ``None`` (the
+            default) the :attr:`write_concern` of this :class:`Collection`
+            is used.
+          - `read_concern` (optional): An instance of
+            :class:`~pymongo.read_concern.ReadConcern`. If ``None`` (the
+            default) the :attr:`read_concern` of this :class:`Collection`
+            is used.
+
+        .. versionadded:: 3.8
+        """
+        return Database(self.client,
+                        self.__name,
+                        codec_options or self.codec_options,
+                        read_preference or self.read_preference,
+                        write_concern or self.write_concern,
+                        read_concern or self.read_concern)
+
     def __eq__(self, other):
         if isinstance(other, Database):
             return (self.__client == other.client and
@@ -361,7 +401,8 @@ class Database(common.BaseObject):
            Removed deprecated argument: options
         """
         with self.__client._tmp_session(session) as s:
-            if name in self.list_collection_names(session=s):
+            if name in self.list_collection_names(
+                    filter={"name": name}, session=s):
                 raise CollectionInvalid("collection %s already exists" % name)
 
             return Collection(self, name, True, codec_options,
@@ -575,6 +616,9 @@ class Database(common.BaseObject):
            :attr:`read_preference` or :attr:`codec_options`. You must use the
            `read_preference` and `codec_options` parameters instead.
 
+        .. note:: :meth:`command` does **not** apply any custom TypeDecoders
+           when decoding the command response.
+
         .. versionchanged:: 3.6
            Added ``session`` parameter.
 
@@ -651,12 +695,14 @@ class Database(common.BaseObject):
             cursor = self._command(sock_info, cmd, slave_okay)["cursor"]
             return CommandCursor(coll, cursor, sock_info.address)
 
-    def list_collections(self, session=None, **kwargs):
+    def list_collections(self, session=None, filter=None, **kwargs):
         """Get a cursor over the collectons of this database.
 
         :Parameters:
           - `session` (optional): a
             :class:`~pymongo.client_session.ClientSession`.
+          - `filter` (optional):  A query document to filter the list of
+            collections returned from the listCollections command.
           - `**kwargs` (optional): Optional parameters of the
             `listCollections command
             <https://docs.mongodb.com/manual/reference/command/listCollections/>`_
@@ -668,6 +714,8 @@ class Database(common.BaseObject):
 
         .. versionadded:: 3.6
         """
+        if filter is not None:
+            kwargs['filter'] = filter
         read_pref = ((session and session._txn_read_preference())
                      or ReadPreference.PRIMARY)
         with self.__client._socket_for_reads(
@@ -676,18 +724,42 @@ class Database(common.BaseObject):
                 sock_info, slave_okay, session, read_preference=read_pref,
                 **kwargs)
 
-    def list_collection_names(self, session=None):
+    def list_collection_names(self, session=None, filter=None, **kwargs):
         """Get a list of all the collection names in this database.
 
+        For example, to list all non-system collections::
+
+            filter = {"name": {"$regex": r"^(?!system\.)"}}
+            db.list_collection_names(filter=filter)
+
         :Parameters:
           - `session` (optional): a
             :class:`~pymongo.client_session.ClientSession`.
+          - `filter` (optional):  A query document to filter the list of
+            collections returned from the listCollections command.
+          - `**kwargs` (optional): Optional parameters of the
+            `listCollections command
+            <https://docs.mongodb.com/manual/reference/command/listCollections/>`_
+            can be passed as keyword arguments to this method. The supported
+            options differ by server version.
+
+        .. versionchanged:: 3.8
+           Added the ``filter`` and ``**kwargs`` parameters.
 
         .. versionadded:: 3.6
         """
+        if filter is None:
+            kwargs["nameOnly"] = True
+        else:
+            # The enumerate collections spec states that "drivers MUST NOT set
+            # nameOnly if a filter specifies any keys other than name."
+            common.validate_is_mapping("filter", filter)
+            kwargs["filter"] = filter
+            if not filter or (len(filter) == 1 and "name" in filter):
+                kwargs["nameOnly"] = True
+
         return [result["name"]
-                for result in self.list_collections(session=session,
-                                                    nameOnly=True)]
+                for result in self.list_collections(session=session, **kwargs)]
 
     def collection_names(self, include_system_collections=True,
                          session=None):
@@ -828,9 +900,9 @@ class Database(common.BaseObject):
         cmd = SON([("currentOp", 1), ("$all", include_all)])
         with self.__client._socket_for_writes() as sock_info:
             if sock_info.max_wire_version >= 4:
-                with self.__client._tmp_session(session) as s:
-                    return sock_info.command("admin", cmd, session=s,
-                                             client=self.__client)
+                return self.__client.admin._command(
+                    sock_info, cmd, codec_options=self.codec_options,
+                    session=session)
             else:
                 spec = {"$all": True} if include_all else {}
                 return _first_batch(sock_info, "admin", "$cmd.sys.inprog",
@@ -1031,7 +1103,7 @@ class Database(common.BaseObject):
 
             opts["roles"] = [self._default_role(read_only)]
 
-        elif read_only:
+        if read_only:
             warnings.warn("The read_only option is deprecated in MongoDB "
                           ">= 2.6, use 'roles' instead", DeprecationWarning)
 
